﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ethernet_data
{
    class reader
    {
        public static reader_IpAddress Ip_Address;
        public struct reader_IpAddress
        {
            public string name;
            public string IP;
            public string SubnetMask;
            public string Gateway;
            public string MAC;
        };

        static UInt16 cncrc_16 = 0x8005;
        static UInt32[] table_cre = new UInt32[256];

        public static byte[] GetReader_list()
        {
            byte[] b = new byte[208];
            int i = 0;
            b[i++] = 0x7f;
            b[i++] = 0x10;
            b[i++] = 0x03;
            b[i++] = 0x00;
            b[i++] = 0xD0;
            b[i++] = 0x12;
            b[i++] = 0x06;
            b[207] = 0x0D;
            long crc = crc_16(b,208);
            byte L = (byte)(crc << 8 >> 8);
            byte H = (byte)(crc>> 8);
            b[205] =H;
            b[206] =L;
            return b;
        }
        public static byte[] SETReader(string IP, string SubnetMask, string Gateway, string MAC)
        {
            byte[] b = new byte[208];
            int i = 0;
            b[i++] = 0x7f;
            b[i++] = 0x10;
            b[i++] = 0x05;
            b[i++] = 0x00;
            b[i++] = 0xD0;
            b[i++] = 0x12;
            b[i++] = 0x06;
            i += 64;
            byte[] byte_MAC = Array.ConvertAll(MAC.Split('-'), new Converter<string, byte>(StringToByte));
            Array.Copy(byte_MAC, 0, b, i, 6); //6字节 MAC
            i += 6;

            byte[] byte_IP = Array.ConvertAll(IP.Split('.'), new Converter<string, byte>(StringToByte));
            Array.Copy(byte_IP, 0, b, i, 4); //4字节 IP
            i += 4;

            byte[] byte_SubnetMask = Array.ConvertAll(SubnetMask.Split('.'), new Converter<string, byte>(StringToByte));
            Array.Copy(byte_SubnetMask, 0, b, i, 4); //4字节 子网掩码
            i += 4;

            byte[] byte_Gateway = Array.ConvertAll(Gateway.Split('.'), new Converter<string, byte>(StringToByte));
            Array.Copy(byte_Gateway, 0, b, i, 4); //4字节 网关

            b[207] = 0x0D;
            long crc = crc_16(b, 208);
            byte L = (byte)(crc << 8 >> 8);
            byte H = (byte)(crc >> 8);
            b[205] = H;
            b[206] = L;
            return b;
        }
        public static byte[] Set_WorkMode(string IP, string MAC)
        {
            byte[] b = new byte[208];
            int i = 0;
            b[i++] = 0x7f;
            b[i++] = 0x10;
            b[i++] = 0x1E;
            b[i++] = 0x00;
            b[i++] = 0xD0;
            b[i++] = 0x12;
            b[i++] = 0x06;
            i += 64;
            byte[] byte_MAC = Array.ConvertAll(MAC.Split('-'), new Converter<string, byte>(StringToByte));
            Array.Copy(byte_MAC, 0, b, i, 6); //6字节 MAC
            i += 6;
            b[i++] = 0x00;
            b[i++] = 0x01;
            b[i++] = 0x00; // 工作模式0:client 1:server 2:udp 3:tcpauto
            b[i++] = 0x75;
            b[i++] = 0x30;  //0x7530=30000; 本地端口(设备 DL9230)
            b[i++] = 0x0; //域名使能。

            byte[] byte_IP = Array.ConvertAll(IP.Split('.'), new Converter<string, byte>(StringToByte));
            Array.Copy(byte_IP, 0, b, i, 4); //4字节 IP
            i += 30;
            
            b[i++] = 0x27;
            b[i++] = 0x10;  //0x2710=10000; 目的端口(电脑)

            b[i++] = 0x1;  //1：立即连接
            b[i++] = 0x01;
            b[i++] = 0x2C;  //0X012C=300秒

            b[207] = 0x0D;
            long crc = crc_16(b, 208);
            byte L = (byte)(crc << 8 >> 8);
            byte H = (byte)(crc >> 8);
            b[205] = H;
            b[206] = L;
            return b;
        }

        private static byte StringTohex(string str)
        {
            return Convert.ToByte(str, 16);
        }
        private  static byte StringToByte(string str)
        {
           return Convert.ToByte(str, 10);
        }
        
        private static void bulidtable16(int apoly)
        {
            UInt16 ndata;
            UInt16 naccum;
            for (UInt16 i = 0; i < 256; i++)
            {
                ndata =(UInt16)(i << 8);
                naccum = 0;
                for (UInt16 j = 0; j < 8; j++)
                {
                    if (((ndata ^ naccum) & 0x8000)>0)
                        naccum =(UInt16)((naccum << 1) ^ apoly);
                    else
                        naccum <<= 1;
                    ndata <<= 1;
                }
                table_cre[i] =(UInt32) naccum;
            }
        }
        private static UInt16 crc_16(byte[] adata, UInt16 asize)
        {
            UInt16 naccum = 0;
            bulidtable16(cncrc_16);
            for (int i = 0; i < asize; i++)
            { 
                naccum=(UInt16)((naccum <<8) ^ table_cre[(naccum >>8 ) ^ adata[i]]);
            }
            return naccum;
        }


    }
}
